@@ -4,6 +4,26 @@ set_RGEOS_HANDLE <- function(handle) {
     assign("GEOSptr", handle, envir=.RGEOS_HANDLE)
 }
 
+set_RGEOS_DENSE <- function(value) {
+    stopifnot(is.logical(value))
+    stopifnot(length(value) == 1)
+    assign("returnDense", value, envir=.RGEOS_HANDLE)
+}
+
+get_RGEOS_DENSE <- function() {
+    get("returnDense", envir=.RGEOS_HANDLE)
+}
+
+set_RGEOS_STR <- function(value) {
+    stopifnot(is.logical(value))
+    stopifnot(length(value) == 1)
+    assign("STRsubset", value, envir=.RGEOS_HANDLE)
+}
+
+get_RGEOS_STR <- function() {
+    get("STRsubset", envir=.RGEOS_HANDLE)
+}
+
 init_RGEOS <- function() {
     .Call('rgeos_Init', PACKAGE="rgeos")
 }
@@ -31,7 +51,8 @@ version_GEOS0 <- function() {
   assign("do_poly_check", TRUE, envir=.RGEOS_HANDLE)
   assign("both_poly", FALSE, envir=.RGEOS_HANDLE)
   assign("drop_not_poly", FALSE, envir=.RGEOS_HANDLE)
-
+  assign("returnDense", TRUE, envir=.RGEOS_HANDLE)
+  assign("STRsubset", FALSE, envir=.RGEOS_HANDLE)
 }
 
 .onAttach <- function(lib, pkg) {
